// solves 2-D Laplace equation using a relaxation scheme
#include <stdio.h>
#include <math.h>
#include <float.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
int main() {
  
   unsigned n = 100, n2 = n+2, maxIter=100000, i, j, iter = 0; 
   double T[n2][n2], Tnew[n2][n2];
   double tol=0.0001, var = DBL_MAX, top = 100.0;
   int itemsread;
   FILE *fout;

   time_t startTime = clock();

   // set boundary conditions

     for (i=0; i<=n; ++i)
         for (j=0; j<=n; ++j)
            T[i][j] = 0.0;

   for (i=1; i<=n; i++) {
      T[n+1][i] = i * top / (n+1);
      T[i][n+1] = i * top / (n+1);
   }

   while(var > tol && iter <= maxIter) {
      ++iter;
      var = 0.0;
      for (i=1; i<=n; ++i)
         for (j=1; j<=n; ++j) {
            Tnew[i][j] = 0.25*( T[i-1][j] + T[i+1][j]
                                + T[i][j-1] + T[i][j+1] );

            var = fmax(var, fabs(Tnew[i][j] - T[i][j]));
         }

         if (iter%100 == 0)
            printf("iter: %8u, variation = %12.4lE\n", iter, var);
    for (i=1; i<=n; ++i)
         for (j=1; j<=n; ++j) 
            T[i][j]=Tnew[i][j];
   }

   double endTime = (clock() - startTime) / (double) CLOCKS_PER_SEC;

   printf("Elapsed time (s) = %.2lf\n", endTime);
   printf("Mesh size: %u\n", n);
   printf("Stopped at iteration: %u\n", iter);
   printf("Maximum error: %lE\n", var);

   // saving results to file
   fout = fopen("results", "w");
   if (fout == NULL) {
      perror("results");
      exit(-1);
   }
   for (i=1; i<=n; ++i)
      for (j=1; j<=n; ++j)
         fprintf(fout, "%8u %8u %18.9lE\n", i, j, T[i][j]);
   fclose(fout);
   return 0;
}

